# Configuration

# Config files

Configuration is done using YAML and should be parsed by any tools we use for building the project. This
means gulp, fabric and event the Taskfile itself need to provide the configuration for the tasks we built.

Configuration basically is done using two configuration files:

### config.yml

This file includes the main project configuration and is shared between all developers of the project.
It may therefore not include any system-specifix paths and options that may differ from developer to
developer.

Common usage may include Sass paths, PostCSS configuration, deployment details and so on. You may not
use the key "local" inside the generic configuration file.

### local.yml

The local configuration should never be put into versioning and are purely for usage by each developer.
All configurations options here are handled separately inside our tasks, while tasks may allow
deveopers to override certain global configuration options (local.option if isset(local.option) else
config.option).

Please make sure to understand that configuration options NEVER get merged together. The tasks itself
need to decide whether the user may override options inside local.yml.

The `local.yml` will be generated using `local.example.yml`on the initial `be install`. This way you get
an usable exmaple configuration whenever necessary.

## Taskfile configuration access

Within the Taskfile b5 will take care of configuration loading and provide all options using normal bash
variables. You may dump the variable information using the provided "config_dump" function inside your
Taskfile. Example config.xml:

```bash
task:debug_config() {
    b5:config_dump
}
```

In general all options are broken down to variables like this (prefixed with "CONFIG_"):

```yaml
application:
  web_path: ../web
  foo: bar
```

gets transformed into

```bash
CONFIG_application_web_path=("../web")
CONFIG_application_foo=("bar")
```

Configuration options from local.yml are prefixed with "CONFIG_local_", which could result in the following:

```bash
CONFIG_local_foo=("bar")
```
