# Modules

## About modules

b5 provides some common modules to allow for easy usage of generic tasks, helper functions and more. Each
module is loaded by normal `source` command by the bash and thus is able to modify the command flow. Modules
must use the following naming schema for avoiding conflicts.

### Loading modules

Inside your Taskfile you may use `b5:module_load $MODULE_NAME` to load modules. Example:

```bash
b5:module_load composer
```

### Naming schema

Example module: "example"

```bash
# Values are prefixed with the module name in all upper case
EXAMPLE_VARIABLE="some value"

# Helper functions are prefixed with the module name 
example:function() {
    echo "Did something"
}

# Modules may define tasks
task:example() {
    example:function
}
```

This is important when calling module functions and for understanding how to write your own modules.

### Hooks

Some generic tasks use module hooks to call all modules. This is currently used for the following tasks:
* install
* update
* clean

By using `b5:module_run_all $HOOK` you are able to use this mechanism by yourself. b5 will then call
`$MODULENAME:$HOOK` in each loaded module (for example: `composer:install`).

The provided install-task looks like this:

```bash
task:install() {
    b5:module_run_all install
}
```

### Example Taskfile

```bash
#!/usr/bin/env bash
# b5 Taskfile, see https://git.team23.de/build/b5 for details

b5:module_load composer  # Provides composer task and install/update hooks

task:example() {
    echo "This is a minimal example task"
}
```

You may now use `b5 install` to install your composer packages, `b5 update` for updating the installed packages
and `b5 composer install …` or any other composer command for using composer.

## Existing modules

See source of module for details.

### bashtpl

Module for generating settings files from templates. Provides helper functions for updating the settings files.

### composer

Handling of composer dependencies and install/update hooks.

### django

Depends on: virtualenv

Will help you use Django, provides update-hook for applying DB migrations.

### docker

Module for setting up docker containers using `docker-compose`.

### docker_django

Helper module for running `manage.py` inside the docker container. Uses "web" container by default.

### drupal

Installs drush to `$PROJECT_PATH/_drush` (by default) and helps you use drush for project handling.

### fabric_compat

Will call `fab -l` to know all defined fabric tasks and provide appropriate b5 tasks.  
(For backwards compatibility and simple b5 setup of old projects)

Initial Taskfile for old projects may be:

```bash
#!/usr/bin/env bash
# b5 Taskfile, see https://git.team23.de/build/b5 for details

b5:module_load fabric_compat
```

### npm

Will install yout node packages and helps you use npm.

### typo3

Depends on: composer

Provides tasks for `typo3` and `typo3cms` command line tools.

### virtualenv

Sets up a virtualenv for python development and provides tools for calling commands using the virtualenv.
